@@ -90,12 +90,17 @@ default_settings = dict(
                 definition='Root urlconf for the /accounts endpoints'),
     A2_REGISTRATION_FORM_CLASS = Setting(default='authentic2.registration_backend.forms.RegistrationForm',
                 definition='Default registration form'),
-    A2_REGISTRATION_SET_PASSWORD_FORM_CLASS = Setting(default='django.contrib.auth.forms.SetPasswordForm',
+    A2_REGISTRATION_SET_PASSWORD_FORM_CLASS = Setting(default='authentic2.registration_backend.forms.SetPasswordForm',
                 definition='Default set password form'),
-    A2_REGISTRATION_CHANGE_PASSWORD_FORM_CLASS = Setting(default='django.contrib.auth.forms.PasswordChangeForm',
+    A2_REGISTRATION_CHANGE_PASSWORD_FORM_CLASS = Setting(default='authentic2.registration_backend.forms.PasswordChangeForm',
                 definition='Default change password form'),
     A2_REGISTRATION_CAN_DELETE_ACCOUNT = Setting(default=True,
                 definition='Can user self delete their account and all their data'),
+    A2_REGISTRATION_CAN_CHANGE_PASSWORD = Setting(default=True, definition='Allow user to change its own password'),
+    A2_PROFILE_CAN_CHANGE_EMAIL = Setting(default=True,
+                definition='Can user self change their email'),
+    A2_PROFILE_CAN_EDIT_PROFILE = Setting(default=True,
+                definition='Can user self edit their profile'),
     A2_HOMEPAGE_URL = Setting(default=None, definition='IdP has no homepage, '
         'redirect to this one.'),
     A2_CAN_RESET_PASSWORD = Setting(default=True, definition='Allow online reset of passwords'),
@@ -119,6 +124,11 @@ default_settings = dict(
     A2_OPENED_SESSION_COOKIE_NAME=Setting(default='A2_OPENED_SESSION', definition='Authentic session open'),
     A2_OPENED_SESSION_COOKIE_DOMAIN=Setting(default=None),
     A2_ATTRIBUTE_KINDS=Setting(default=[], definition='List of other attribute kinds'),
+    A2_VALIDATE_EMAIL=Setting(default=False, definition='Validate user email domain'),
+    A2_PASSWORD_POLICY_MIN_CLASSES=Setting(default=3, definition='Minimum number of characters classes to be present in passwords'),
+    A2_PASSWORD_POLICY_MIN_LENGTH=Setting(default=6, definition='Minimum number of characters in a password'),
+    A2_AUTH_PASSWORD_ENABLE=Setting(default=True, definition='Activate login/password authentication', names=('AUTH_PASSWORD',)),
+    PUSH_PROFILE_UPDATES=Setting(default=False, definition='Push profile update to linked services'),
 )
 
 app_settings = AppSettings(default_settings)
