@@ -1,19 +1,15 @@
 # encoding: utf-8
-import datetime
 from south.db import db
 from south.v2 import SchemaMigration
-from django.db import models
 
 class Migration(SchemaMigration):
 
     def forwards(self, orm):
-        
         # Changing field 'AuthenticationEvent.nonce'
         db.alter_column('auth2_auth_authenticationevent', 'nonce', self.gf('django.db.models.fields.CharField')(max_length=255))
 
 
     def backwards(self, orm):
-        
         # Changing field 'AuthenticationEvent.nonce'
         db.alter_column('auth2_auth_authenticationevent', 'nonce', self.gf('django.db.models.fields.CharField')(max_length=20))
 
