@@ -9,7 +9,6 @@ import logging
 import random
 import pickle
 import base64
-import hashlib
 import urllib
 import itertools
 
@@ -29,11 +28,10 @@ try:
 except ImportError:
     pass
 
-from authentic2.cache import get_shared_cache
 from authentic2.decorators import to_list
 from authentic2.compat import get_user_model, commit_on_success
 from authentic2.models import UserExternalId
-
+from authentic2.middleware import StoreRequestMiddleware
 
 _DEFAULTS = {
     'binddn': None,
@@ -94,12 +92,20 @@ _DEFAULTS = {
     'external_id_tuples': (('dn:noquote',),),
     # keep password around so that Django authentification also work
     'clean_external_id_on_update': True,
+    # Conserve the passsword in the Django User object
     'keep_password': True,
+    # Only authenticate users coming from the corresponding realm
     'limit_to_realm': True,
+    # Assign users mandatorily to some groups
+    'set_mandatory_groups': (),
 }
 
 _REQUIRED = ('url', 'basedn')
 _TO_ITERABLE = ('url', 'groupsu', 'groupstaff', 'groupactive')
+_TO_LOWERCASE = ('fname_field', 'lname_field', 'email_field', 'attributes',
+    'mandatory_attributes_values')
+_VALID_CONFIG_KEYS = list(set(_REQUIRED).union(set(_DEFAULTS)))
+
 
 def get_connection(block, credentials=()):
     if not block['url']:
@@ -143,19 +149,22 @@ def modify_password(conn, block, dn, old_password, new_password):
     conn.modify_s(dn, ldap.modlist.modifyModlist(old_entry, new_entry))
     log.debug('modified password for dn %r', dn)
 
-def unicode_dict_of_list(attributes, encoding='utf-8'):
+def normalize_ldap_results(attributes, encoding='utf-8'):
+    new_attributes = {}
     for key in attributes:
         try:
-            attributes[key] = map(lambda x: unicode(x, encoding), attributes[key])
+            new_attributes[key.lower()] = map(lambda x: unicode(x, encoding), attributes[key])
         except UnicodeDecodeError:
-            log.warning('unable to decode attribute %r to UTF-8', key)
-    return attributes
+            log.debug('unable to decode attribute %r as UTF-8, converting to base64', key)
+            new_attributes[key.lower()] = map(base64.b64encode, attributes[key])
+    return new_attributes
 
 class LDAPException(Exception):
     pass
 
 class LDAPUser(get_user_model()):
     attributes = {}
+    SESSION_PASSWORD_KEY = 'ldap-password-cached'
 
     class Meta:
         proxy = True
@@ -168,33 +177,30 @@ class LDAPUser(get_user_model()):
         self.transient = transient
 
     def set_ldap_password(self, password):
-        shared_cache = get_shared_cache('ldap')
-        shared_cache.set(self.get_ldap_password_cache_key(), password, 86400)
+        request = StoreRequestMiddleware.get_request()
+        cache = request.session.setdefault(self.SESSION_PASSWORD_KEY, {})
+        cache[self.dn] = password
+        request.session.modified = True
 
-    def get_cache_key(self):
-        return hashlib.md5(self.dn).hexdigest()
-
-    def get_ldap_password_cache_key(self):
-        return 'ldap-password-{0}'.format(self.get_cache_key())
-
-    def get_password(self):
-        shared_cache = get_shared_cache('ldap')
-        password = shared_cache.get(self.get_ldap_password_cache_key())
+    def get_ldap_password(self):
+        request = StoreRequestMiddleware.get_request()
+        cache = request.session.setdefault(self.SESSION_PASSWORD_KEY, {})
+        password = cache.get(self.dn)
         if password is None:
             raise LDAPException('missing password for dn %r', self.dn)
         return password
 
     def set_password(self, new_password):
-        old_password = self.get_password()
+        old_password = self.get_ldap_password()
         if old_password != new_password:
             conn = self.get_connection()
-            modify_password(conn, self.block, self.dn, self.get_password(),
+            modify_password(conn, self.block, self.dn, self.get_ldap_password(),
                     new_password)
         self.set_ldap_password(new_password)
         super(LDAPUser, self).set_password(new_password)
 
     def get_connection(self):
-        return get_connection(self.block, (self.dn, self.get_password()))
+        return get_connection(self.block, (self.dn, self.get_ldap_password()))
 
     def get_attributes(self):
         conn = self.get_connection()
@@ -223,12 +229,20 @@ class LDAPBackend(object):
 
     @classmethod
     def get_config(self):
+        if not getattr(settings, 'LDAP_AUTH_SETTINGS', []):
+            return []
         if isinstance(settings.LDAP_AUTH_SETTINGS[0], dict):
             blocks = settings.LDAP_AUTH_SETTINGS
         else:
             blocks = (self._parse_simple_config(),)
         # First get our configuration into a standard format
         for block in blocks:
+            for key in block:
+                if not key in _VALID_CONFIG_KEYS:
+                    raise ImproperlyConfigured(
+                        ('"{}" : invalid LDAP_AUTH_SETTINGS key, '
+                        +'available are {}').format(key, _VALID_CONFIG_KEYS))
+
             for r in _REQUIRED:
                 if r not in block:
                     raise ImproperlyConfigured('LDAP_AUTH_SETTINGS: missing required configuration option %r' % r)
@@ -261,7 +275,32 @@ class LDAPBackend(object):
             for i in _TO_ITERABLE:
                 if isinstance(block[i], basestring):
                     block[i] = (block[i],)
-
+            # lowercase LDAP attribute names
+            for key in _TO_LOWERCASE:
+                # we handle strings, list of strings and list of list or tuple whose first element is a string
+                if isinstance(block[key], basestring):
+                    block[key] = block[key].lower()
+                elif isinstance(block[key], (list, tuple)):
+                    new_seq = []
+                    for elt in block[key]:
+                        if isinstance(elt, basestring):
+                            elt = elt.lower()
+                        elif isinstance(elt, (list, tuple)):
+                            elt = list(elt)
+                            elt[0] = elt[0].lower()
+                            elt = tuple(elt)
+                        new_seq.append(elt)
+                    block[key] = tuple(new_seq)
+                elif isinstance(block[key], dict):
+                    newdict = {}
+                    for subkey in block[key]:
+                        newdict[subkey.lower()] = block[subkey]
+                    block[key] = newdict
+                else:
+                    raise NotImplementedError('LDAP setting %r cannot be '
+                            'converted to lowercase '
+                            'setting, its type is %r'
+                            % (key, type(block[key])))
             # Want to randomize our access, otherwise what's the point of having multiple servers?
             block['url'] = list(block['url'])
             if block['shuffle_replicas']:
@@ -346,7 +385,6 @@ class LDAPBackend(object):
                             break
                         continue
                     except ldap.LDAPError, e:
-                        raise
                         log.error('user lookup failed: with query %r got error '
                                 '%s', username, query, e)
                         continue
@@ -578,8 +616,7 @@ class LDAPBackend(object):
         except ldap.LDAPError:
             log.exception('unable to retrieve attributes of dn %r', dn)
             return {}
-        attribute_map = results[0][1]
-        attribute_map = unicode_dict_of_list(attribute_map)
+        attribute_map = normalize_ldap_results(results[0][1])
         # add mandatory attributes
         for key, mandatory_values in mandatory_attributes_values.iteritems():
             key = str(key)
