@@ -23,6 +23,7 @@ import xml.etree.cElementTree as ctree
 import hashlib
 import random
 import string
+from functools import wraps
 
 import lasso
 from django.core.urlresolvers import reverse
@@ -30,14 +31,15 @@ from django.contrib.auth.decorators import login_required
 from django.core.exceptions import ObjectDoesNotExist
 from django.http import HttpResponse, HttpResponseRedirect, \
     HttpResponseForbidden, HttpResponseBadRequest, Http404
-from django.utils.translation import ugettext as _
+from django.utils.translation import ugettext as _, ugettext_noop as N_
 from django.views.decorators.csrf import csrf_exempt
 from django.views.decorators.cache import never_cache
-from django.contrib.auth import BACKEND_SESSION_KEY
+from django.contrib.auth import BACKEND_SESSION_KEY, REDIRECT_FIELD_NAME
 from django.conf import settings
 from django.utils.encoding import smart_unicode
 from django.contrib.auth import load_backend
-from django.shortcuts import render
+from django.shortcuts import render, redirect
+from django.contrib import messages
 
 
 from authentic2.compat import get_user_model
@@ -96,7 +98,6 @@ metadata_map = (
 )
 metadata_options = {'key': settings.SAML_SIGNATURE_PUBLIC_KEY}
 
-
 @cache_and_validate(settings.LOCAL_METADATA_CACHE_TIMEOUT)
 def metadata(request):
     '''Endpoint to retrieve the metadata file'''
@@ -104,6 +105,19 @@ def metadata(request):
     return HttpResponse(get_metadata(request, request.path),
             mimetype='text/xml')
 
+def log_assert(func, exception_classes=(AssertionError,)):
+    '''Convert assertion errors to warning logs and report them to the user
+       through the messages framework.
+
+       Returns a redirect to homepage or the `next` query parameter.
+    '''
+    @wraps(func)
+    def f(request, *args, **kwargs):
+        try:
+            return func(request, *args, **kwargs)
+        except exception_classes, e:
+            return error_redirect(request, e.message)
+    return f
 
 #####
 # SSO
@@ -307,6 +321,7 @@ def build_assertion(request, login, nid_format='transient', attributes=None):
             notBefore.isoformat() + 'Z',
             notOnOrAfter.isoformat() + 'Z')
     assertion = login.assertion
+    assertion.conditions.notOnOrAfter = notOnOrAfter.isoformat() + 'Z'
     logger.debug("assertion building in progress %s" \
         % assertion.dump())
     logger.debug("fill assertion")
@@ -345,6 +360,7 @@ def build_assertion(request, login, nid_format='transient', attributes=None):
 
 @never_cache
 @csrf_exempt
+@log_assert
 def sso(request):
     """Endpoint for receiving saml2:AuthnRequests by POST, Redirect or SOAP.
        For SOAP a session must be established previously through the login
@@ -374,7 +390,7 @@ def sso(request):
             break
         except (lasso.ProfileInvalidMsgError,
             lasso.ProfileMissingIssuerError,), e:
-            logger.error('invalid message for WebSSO profile with '
+            logger.warning('invalid message for WebSSO profile with '
                           'HTTP-Redirect binding: %r exception: %s' \
                           % (message, e),
                           extra={'request': request})
@@ -385,18 +401,18 @@ def sso(request):
             log_info_authn_request_details(login)
             message = _("SAMLv2 Single Sign On: the request cannot be "
                 "answered because no valid protocol binding could be found")
-            logger.error("the request cannot be answered because no "
+            logger.warning("the request cannot be answered because no "
                 "valid protocol binding could be found")
             login.response.status.statusMessage = 'No valid protocol binding could be found'
             return HttpResponseBadRequest(message)
         except lasso.ProviderMissingPublicKeyError, e:
             log_info_authn_request_details(login)
-            logger.error('no public key found: %s', e)
+            logger.warning('no public key found: %s', e)
             login.response.status.statusMessage = 'The public key is unknown'
             return return_login_response(request, login)
         except lasso.DsError, e:
             log_info_authn_request_details(login)
-            logger.error('digital signature treatment error: %s', e)
+            logger.warning('digital signature treatment error: %s', e)
             login.response.status.statusMessage = 'Signature validation failed'
             return return_login_response(request, login)
         except (lasso.ServerProviderNotFoundError,
@@ -418,7 +434,6 @@ def sso(request):
             else:
                 policy = get_sp_options_policy(provider_loaded)
                 if not policy:
-                    logger.error('No policy defined')
                     return error_page(request, _('sso: No SP policy defined'),
                         logger=logger, warning=True)
                 logger.info('provider %s loaded with success' \
@@ -429,7 +444,7 @@ def sso(request):
                     provider_loaded.service_provider.policy \
                             .authn_request_signature_check_hint)
     if signed and not check_destination(request, login.request):
-        logger.error('wrong or absent destination')
+        logger.warning('wrong or absent destination')
         return return_login_error(request, login,
                 AUTHENTIC_STATUS_CODE_MISSING_DESTINATION)
     # Check NameIDPolicy or force the NameIDPolicy
@@ -537,8 +552,11 @@ def continue_sso(request):
     if not nonce:
         logger.warning('nonce not found')
         return HttpResponseBadRequest()
-    login_dump, consent_obtained, save, nid_format = \
-            get_and_delete_key_values(nonce)
+    try:
+        login_dump, consent_obtained, save, nid_format = \
+                get_and_delete_key_values(nonce)
+    except ValueError:
+        return error_redirect(request, N_('request has expired'))
     server = create_server(request)
     # Work Around for lasso < 2.3.6
     login_dump = login_dump.replace('<Login ', '<lasso:Login ') \
@@ -602,6 +620,14 @@ def sso_after_process_request(request, login, consent_obtained=False,
         logger.info('login required')
         return need_login(request, login, save, nid_format)
 
+    # No user is authenticated and passive is True, deny request
+    if passive and user.is_anonymous():
+        logger.info("%r - no user connected and passive request, returning "
+                "NoPassive", nonce)
+        set_saml2_response_responder_status_code(login.response,
+                lasso.SAML2_STATUS_CODE_NO_PASSIVE)
+        return finish_sso(request, login)
+
     #Deal with transient users
     transient_user = False
     # XXX: Deal with all kind of transient users
@@ -688,15 +714,11 @@ def sso_after_process_request(request, login, consent_obtained=False,
     provider = load_provider(request, login.remoteProviderId,
         server=login.server)
     if not provider:
-        logger.info(''
-            'sso for an unknown provider %s' % login.remoteProviderId)
         return error_page(request,
             _('Provider %s is unknown') % login.remoteProviderId,
             logger=logger)
     saml_policy = get_sp_options_policy(provider)
     if not saml_policy:
-        logger.error('No policy defined for '
-            'provider %s' % login.remoteProviderId)
         return error_page(request, _('No service provider policy defined'),
             logger=logger)
 
@@ -958,66 +980,45 @@ def check_delegated_authentication_permission(request):
 @never_cache
 @csrf_exempt
 @login_required
-def idp_sso(request, provider_id=None, user_id=None, nid_format=None,
-        save=True, return_profile=False):
+def idp_sso(request, provider_id=None, save=True, return_profile=False):
     '''Initiate an SSO toward provider_id without a prior AuthnRequest
     '''
     User = get_user_model()
-    if request.method == 'GET':
-        logger.info('to initiate a sso we need a post form')
-        return error_page(request,
-            _('Error trying to initiate a single sign on'), logger=logger)
     if not provider_id:
-        provider_id = request.POST.get('provider_id')
+        provider_id = request.REQUEST.get('provider_id')
     if not provider_id:
-        logger.info('to initiate a sso we need a provider_id')
-        return error_page(request,
-            _('A provider identifier was not provided'), logger=logger)
-    logger.info('sso initiated with %(provider_id)s' \
-        % {'provider_id': provider_id})
-    if user_id:
-        logger.info('sso as %s' % user_id)
+        return error_redirect(request,
+                N_('missing provider identifier'))
     server = create_server(request)
     login = lasso.Login(server)
     liberty_provider = load_provider(request, provider_id,
         server=login.server)
     if not liberty_provider:
-        logger.info('sso for an unknown provider %s' % provider_id)
-        return error_page(request, _('Provider %s is unknown') % provider_id,
-            logger=logger)
-    if user_id:
-        user = User.get(id=user_id)
+        return error_redirect(request, N_('provider %r is unknown'), provider_id)
+    username = request.REQUEST.get('username')
+    if username:
         if not check_delegated_authentication_permission(request):
-            logger.warning('%r tried to log as %r on %r but was '
-                'forbidden' % (request.user, user, provider_id))
-            return HttpResponseForbidden('You must be superuser to log as '
-                'another user')
+            return error_redirect(request,
+                    N_('%r tried to log as %r on %r but was forbidden'),
+                   request.user, username, provider_id)
+        try:
+            user = User.objects.get_by_natural_key(username=username)
+        except User.DoesNotExist:
+            return error_redirect(request,
+                    N_('you cannot login as %r as it does not exist'), username)
     else:
         user = request.user
-        logger.info('sso by %r' % user)
+    logger.info('sso by %r', user)
     policy = get_sp_options_policy(liberty_provider)
-    if nid_format:
-        logger.debug('nameId format is %r' % nid_format)
-        if not nid_format in policy.accepted_name_id_format:
-            logger.error('name id format %r is not supported by %r' \
-                % (nid_format, provider_id))
-            raise Http404('Provider %r does not support this name id format' \
-                % provider_id)
-    if not nid_format:
-        nid_format = policy.default_name_id_format
-        logger.debug('nameId format is %r' % nid_format)
-    if needs_persistence(nid_format):
-        load_federation(request, get_entity_id(request, reverse(metadata)), login, user)
-    logger.debug('federation loaded')
-    login.initIdpInitiatedAuthnRequest(provider_id)
     # Control assertion consumer binding
     if not policy:
-        logger.error('No policy defined, \
-            unable to set protocol binding')
-        return error_page(request, _('idp_sso: No SP policy defined'),
-            logger=logger)
+        return error_redirect(request,
+                N_('missing service provider policy'))
+    nid_format = policy.default_name_id_format
+    if needs_persistence(nid_format):
+        load_federation(request, get_entity_id(request, reverse(metadata)), login, user)
+    login.initIdpInitiatedAuthnRequest(provider_id)
     binding = policy.prefered_assertion_consumer_binding
-    logger.debug('binding is %r' % binding)
     if binding == 'meta':
         pass
     elif binding == 'art':
@@ -1025,14 +1026,17 @@ def idp_sso(request, provider_id=None, user_id=None, nid_format=None,
     elif binding == 'post':
         login.request.protocolBinding = lasso.SAML2_METADATA_BINDING_POST
     else:
-        logger.error('unsupported protocol binding %r' % binding)
-        return error_page(request, _('Server error'), logger=logger)
+        return error_redirect(request,
+                N_('unknown binding %r') % binding)
     # Control nid format policy
     # XXX: if a federation exist, we should use transient
     login.request.nameIdPolicy.format = nidformat_to_saml2_urn(nid_format)
     login.request.nameIdPolicy.allowCreate = True
 
     login.processAuthnRequestMsg(None)
+    logger.debug('nameId %r' % nid_format)
+    logger.debug('binding %r' % binding)
+    logger.info('authentication request initialized toward provider_id %r', provider_id)
 
     return sso_after_process_request(request, login,
             consent_obtained=False, user=user, save=save,
@@ -1072,7 +1076,7 @@ def return_logout_error(request, logout, error):
     # Hack because response is not initialized before
     # buildResponseMsg
     logout.buildResponseMsg()
-    logger.debug('send an error message %s' % error)
+    logger.warning('returned an error message on logout: %s', error)
     provider = LibertyProvider.objects.get(entity_id=logout.remoteProviderId)
     return return_saml2_response(request, logout,
         title=_('You are being redirected to "%s"') % provider.name)
@@ -1382,12 +1386,12 @@ def slo(request):
         provider = LibertyProvider.objects.get(entity_id=logout.remoteProviderId)
         return return_saml2_response(request, logout,
             title=_('You are being redirected to "%s"') % provider.name)
-    except Exception, e:
-        logger.exception('slo %s' % message)
-        return error_page(_('Invalid logout request'), logger=logger)
+    except (lasso.ProfileInvalidMsgError,
+        lasso.ProfileMissingIssuerError), e:
+        return error_page(_('Invalid logout request'), logger=logger, warning=True)
     session_indexes = logout.request.sessionIndexes
     if len(session_indexes) == 0:
-        logger.error('slo received a request from %s without any \
+        logger.warning('slo received a request from %s without any \
             SessionIndex, it is forbidden' % logout.remoteProviderId)
         logout.buildResponseMsg()
         provider = LibertyProvider.objects.get(entity_id=logout.remoteProviderId)
@@ -1399,7 +1403,7 @@ def slo(request):
             logout.server.providerId, logout.remoteProviderId,
             logout.request.nameId, logout.request.sessionIndexes)
     if not all_sessions.exists():
-        logger.error('slo refused, since no session exists with the \
+        logger.warning('slo refused, since no session exists with the \
             requesting provider')
         return return_logout_error(request, logout,
                 AUTHENTIC_STATUS_CODE_UNKNOWN_SESSION)
@@ -1408,8 +1412,12 @@ def slo(request):
     set_session_dump_from_liberty_sessions(logout, [last_session])
     try:
         logout.validateRequest()
+    except lasso.Error, e:
+        logger.warning('logout request validation failed: %s', e)
+        return return_logout_error(request, logout,
+                AUTHENTIC_STATUS_CODE_INTERNAL_SERVER_ERROR)
     except:
-        logger.exception('slo error')
+        logger.exception('internal error')
         return return_logout_error(request, logout,
                 AUTHENTIC_STATUS_CODE_INTERNAL_SERVER_ERROR)
     # Now clean sessions for this provider
@@ -1422,14 +1430,14 @@ def slo(request):
             (reverse(finish_slo), urllib.quote(logout.request.id)))
 
 
-def ko_icon(request):
-    return HttpResponseRedirect('%s/authentic2/images/ko.png' \
-        % settings.STATIC_URL)
+def icon_url(name):
+    return '%s/authentic2/images/%s.png' % (settings.STATIC_URL, name)
 
+def ko_icon(request):
+    return HttpResponseRedirect(icon_url('ko'))
 
 def ok_icon(request):
-    return HttpResponseRedirect('%s/authentic2/images/ok.png' \
-        % settings.STATIC_URL)
+    return HttpResponseRedirect(icon_url('ok'))
 
 
 @never_cache
@@ -1466,6 +1474,7 @@ def idp_slo(request, provider_id=None):
     provider = load_provider(request, provider_id, server=logout.server)
     if not provider:
         logger.error('slo failed to load provider')
+        return redirect_next(request, next) or ko_icon(request)
     policy = get_sp_options_policy(provider)
     if not policy:
         logger.error('No policy found for %s'\
@@ -1520,7 +1529,7 @@ def process_logout_response(request, logout, soap_response, next):
     try:
         logout.processResponseMsg(soap_response)
     except getattr(lasso, 'ProfileRequestDeniedError', lasso.LogoutRequestDeniedError):
-        logger.warning('logout request was denied')
+        logger.warning('%s denied the logout request', logout.remoteProviderId)
         return redirect_next(request, next) or ko_icon(request)
     except:
         logger.exception('\
@@ -1537,20 +1546,20 @@ def process_logout_response(request, logout, soap_response, next):
 
 @never_cache
 def slo_return(request):
-    next = None
     logger.info('return from redirect')
     relay_state = request.REQUEST.get('RelayState')
     if not relay_state:
-        logger.error('slo no relay state in response')
-        return error_page('Missing relay state', logger=logger)
-    else:
-        logger.debug('relay_state %s' % relay_state)
+        return error_redirect(request, N_('slo no relay state in response'), 
+                default_url=icon_url('ko'))
+    logger.debug('relay_state %r', relay_state)
     try:
         logout_dump, provider_id, next = \
             get_and_delete_key_values(relay_state)
-    except:
-        logger.exception('slo bad relay state in response')
-        return error_page('Bad relay state', logger=logger)
+    except ValueError:
+        return error_redirect(request,
+                N_('unknown relay state %r'),
+                relay_state,
+                default_url=icon_url('ko'))
     server = create_server(request)
     logout = lasso.Logout.newFromDump(server, logout_dump)
     provider_id = logout.remoteProviderId
@@ -1560,7 +1569,7 @@ def slo_return(request):
     # FIXME: should use a logout_request_signature_check_hint
     logout.setSignatureVerifyHint(policy.authn_request_signature_check_hint)
     if not load_provider(request, provider_id, server=logout.server):
-        logger.error('slo failed to load provider')
+        logger.warning('failed to load provider %r', provider_id)
     return process_logout_response(request, logout,
         get_saml2_query_request(request), next)
 
@@ -1637,6 +1646,29 @@ def check_destination(request, req_or_res):
     destination = request.build_absolute_uri(request.path)
     result = req_or_res.destination == destination
     if not result:
-        logger.warning('failure, expected: %r got: %r ' \
-            % (destination, req_or_res.destination))
+        logger.warning('failure, expected: %r got: %r ',
+            destination, req_or_res.destination)
     return result
+
+def error_redirect(request, msg, *args, **kwargs):
+    '''Log a warning message, register it with the messages framework, then
+       redirect the user to the homepage.
+
+       It will redirect to Authentic2 homepage unless a next query parameter was used.
+    '''
+    default_kwargs = {
+            'log_level': logging.WARNING,
+            'msg_level': messages.WARNING,
+            'default_url': None,
+    }
+    default_kwargs.update(kwargs)
+    messages.add_message(request, default_kwargs['msg_level'], _(msg) % args)
+    logger.log(default_kwargs['log_level'], msg, *args)
+    next_url = request.GET.get(REDIRECT_FIELD_NAME)
+    if next_url:
+        return HttpResponseRedirect(next_url)
+    default_url = kwargs.get('default_url')
+    if default_url:
+        return HttpResponseRedirect(default_url)
+    else:
+        return redirect('auth_homepage')
