@@ -26,7 +26,6 @@ from authentic2.saml.models import LibertyFederation, LibertyProvider, \
 from authentic2.saml import models
 from authentic2.saml import saml2utils
 
-from authentic2.authsaml2 import signals
 from authentic2.http_utils import get_url
 from .. import nonce
 
@@ -46,7 +45,7 @@ AUTHENTIC_STATUS_CODE_INTERNAL_SERVER_ERROR = AUTHENTIC_STATUS_CODE_NS + \
 AUTHENTIC_STATUS_CODE_UNAUTHORIZED = AUTHENTIC_STATUS_CODE_NS + \
     "Unauthorized"
 
-logger = logging.getLogger('authentic2.saml.common')
+logger = logging.getLogger(__name__)
 
 # timeout for messages and assertions issue instant
 NONCE_TIMEOUT = getattr(settings, 'SAML2_NONCE_TIMEOUT',
@@ -54,18 +53,15 @@ NONCE_TIMEOUT = getattr(settings, 'SAML2_NONCE_TIMEOUT',
 # do we check id on SAML2 messages ?
 CHECKS_ID = getattr(settings, 'SAML2_CHECKS_ID', True)
 
-def get_soap_message(request, on_error_raise = True):
+def get_soap_message(request):
     '''Verify that POST content looks like a SOAP message and returns it'''
-    if request.method != 'POST' or \
-            'text/xml' not in request.META['CONTENT_TYPE']:
-       if on_error_raise:
-           raise Http404('Only SOAP messages here')
-       else:
-           return None
+    assert request.method == 'POST' \
+            and 'CONTENT_TYPE' in request.META \
+            and 'text/xml' in request.META['CONTENT_TYPE'], 'not a SOAP message'
     return request.raw_post_data
 
 def get_http_binding(request):
-    if request.method == 'GET':
+    if request.method in ('GET', 'HEAD'):
         return 'GET'
     elif request.method == 'POST':
         # disambiguate SOAP and form POST
@@ -113,7 +109,10 @@ def create_saml2_server(request, metadata, idp_map=None, sp_map=None, options={}
 
 def get_saml2_post_response(request):
     '''Extract the SAMLRequest field from the POST'''
-    return request.POST.get(lasso.SAML2_FIELD_RESPONSE, '')
+    msg = request.POST.get(lasso.SAML2_FIELD_RESPONSE, '')
+    assert msg is not None, 'no message received'
+    logger.debug('%r: %r', lasso.SAML2_FIELD_RESPONSE, msg)
+    return msg
 
 def get_saml2_post_request(request):
     '''Extract the SAMLRequest field from the POST'''
@@ -139,11 +138,15 @@ def get_saml2_request_message(request):
     '''Return SAMLv2 message whatever the HTTP binding used'''
     binding = get_http_binding(request)
     if binding == 'GET':
-        return get_saml2_query_request(request)
+        msg = get_saml2_query_request(request)
     elif binding == 'POST':
-        return get_saml2_post_request(request)
+        msg = get_saml2_post_request(request)
     elif binding == 'SOAP':
-        return get_saml2_soap_request(request)
+        msg = get_saml2_soap_request(request)
+    else:
+        msg = None
+    assert msg, 'no saml2 request message found'
+    return msg
 
 def return_saml2_response(request, profile, title = ''):
     '''Finish your SAMLv2 views with this method to return a SAML
@@ -158,6 +161,10 @@ def return_saml2_request(request, profile, title = ''):
 def return_saml2(request, profile, field_name, title = ''):
     '''Helper to handle SAMLv2 bindings to emit request and responses'''
     context_instance = RequestContext(request)
+    logger.debug('profile.msgBody: %r', profile.msgBody)
+    logger.debug('profile.msgUrl: %r', profile.msgUrl)
+    logger.debug('profile.msgRelayState: %r', profile.msgRelayState)
+    logger.debug('field_name: %r', field_name)
     if profile.msgBody:
         if profile.msgUrl:
             return render_to_response('saml/post_form.html',{
@@ -344,25 +351,25 @@ for entity id %r are invalid, %s' % (provider_id, e.args))
         s.save()
     return p
 
-def load_provider(request, provider_id, server=None, sp_or_idp='sp',
+def load_provider(request, entity_id, server=None, sp_or_idp='sp',
         autoload=False):
     '''Look up a provider in the database, and verify it handles wanted
        role be it sp or idp.
 
        Arguments:
        request -- the currently handled request
-       provider_id -- the entity ID of the searched provider
+       entity_id -- the entity ID of the searched provider
        Keyword arguments:
        server -- a lasso.Server object into which to load the given provider
        sp_or_idp -- kind of the provider we are looking for, can be 'sp' or 'idp',
        default to 'sp'
     '''
     try:
-        liberty_provider = LibertyProvider.objects.get(entity_id=provider_id)
+        liberty_provider = LibertyProvider.objects.get(entity_id=entity_id)
     except LibertyProvider.DoesNotExist:
         autoload = getattr(settings, 'SAML_METADATA_AUTOLOAD', 'none')
         if autoload and (autoload == 'sp' or autoload == 'both'):
-            liberty_provider = retrieve_metadata_and_create(request, provider_id, sp_or_idp)
+            liberty_provider = retrieve_metadata_and_create(request, entity_id, sp_or_idp)
             if not liberty_provider:
                 return False
         else:
@@ -389,20 +396,19 @@ def load_provider(request, provider_id, server=None, sp_or_idp='sp',
                     liberty_provider.metadata.encode('utf8'))
     else:
         raise Exception('unsupported option sp_or_idp = %r' % sp_or_idp)
-
+    logger.debug('loaded provider %r', entity_id)
     return liberty_provider
 
 # Federation management
 def add_federation(user, login=None, name_id=None, provider_id=None):
-    if not name_id:
-        if not (login and login.nameIdentifier):
-            return None
-        name_id=login.nameIdentifier
+    assert name_id or (login and login.nameIdentifier), 'missing name identifier'
+    name_id = name_id or login.nameIdentifier
     kwargs = models.nameid2kwargs(name_id)
     if provider_id:
         kwargs['idp'] = LibertyProvider.objects.get(entity_id=provider_id).identity_provider
     fed = LibertyFederation(user=user, **kwargs)
     fed.save()
+    logger.debug('federation %r linked to user %r', fed.name_id_content, user)
     return fed
 
 def lookup_federation_by_name_identifier(name_id=None, profile=None):
@@ -438,31 +444,6 @@ def lookup_federation_by_user(user, qualifier):
         return None
     return fed[0]
 
-# List Idp providers - Use from display in templates
-# WARNING: No way for multiple federation by user with a single IdP (is it a problem??)
-def get_idp_list():
-    return LibertyProvider.objects.exclude(identity_provider=None) \
-            .values('entity_id','name')
-
-def get_idp_list_sorted():
-    return LibertyProvider.objects.exclude(identity_provider=None) \
-            .order_by('name').values('entity_id','name')
-
-def get_idp_user_federated_list(request):
-    user = request.user
-    if request.user.is_anonymous():
-        return None
-    return [p for p in get_idp_list() \
-        if lookup_federation_by_user(user, p.entity_id)]
-
-def get_idp_user_not_federated_list(request):
-    user = request.user
-    if request.user.is_anonymous():
-        return None
-    return [p for p in get_idp_list() \
-        if not lookup_federation_by_user(user, p.entity_id)]
-
-
 # The session_index is the "session on the IdP" identifiers
 # One identifier is dedicated for each sp for each user session
 # to not be a factor of linkability between sp
@@ -557,9 +538,7 @@ def get_provider_of_active_session_name(request):
     return urlparse(p.entity_id)[1]
 
 class SOAPException(Exception):
-    url = None
-    def __init__(self, url):
-        self.url = url
+    pass
 
 def soap_call(url, msg, client_cert = None):
     if not client_cert:
@@ -574,26 +553,28 @@ def soap_call(url, msg, client_cert = None):
         host, query = urllib.splithost(url[6:])
         conn = httplib.HTTPSConnection(host,
                 key_file = client_cert, cert_file = client_cert)
-    logger.debug('soap_call: host %s' % host)
-    logger.debug('soap_call: query %s' % query)
-    logger.debug('soap_call: msg %s' % msg)
+    logger.debug('host %s' % host)
+    logger.debug('query %s' % query)
+    logger.debug('msg %s' % msg)
     try:
         conn.request('POST', query, msg, {'Content-Type': 'text/xml'})
         response = conn.getresponse()
     except Exception, err:
         logging.error('SOAP error (on %s): %s' % (url, err))
-        raise SOAPException(url)
-    logger.debug('soap_call: response %s' % str(response))
+        raise SOAPException(url, err)
+    logger.debug('response %s' % str(response))
     try:
         data = response.read()
     except Exception, err:
         logging.error('SOAP error (on %s): %s' % (url, err))
-        raise SOAPException(url)
-    logger.debug('soap_call: data %s' % str(data))
+        raise SOAPException(url, err)
+    logger.debug('data %s' % str(data))
     conn.close()
     if response.status not in (200, 204): # 204 ok for federation termination
         logging.warning('SOAP error (%s) (on %s)' % (response.status, url))
-        raise SOAPException(url)
+        raise SOAPException(url, 'http status code error', response.status)
+    if not data:
+        raise SOAPException(url, 'no content returned')
     return data
 
 def send_soap_request(request, profile):
@@ -724,9 +705,6 @@ def authz_decision_cb(sender, request=None, attributes={},
             _('Your access is denied. At least one attribute does not match.')
     return dic
 
-signals.authz_decision.connect(authz_decision_cb,
-    dispatch_uid='authz_decision_on_attributes')
-
 def get_session_not_on_or_after(assertion):
     '''Extract the minimal value for the SessionNotOnOrAfter found in the given
        assertion AuthenticationStatement(s).
