@@ -3,6 +3,7 @@ import os
 import json
 import glob
 import re
+import warnings
 
 from django.core.exceptions import ImproperlyConfigured
 from django.conf import global_settings
@@ -25,9 +26,9 @@ def to_boolean(name, default=True):
         i = int(value)
         return bool(i)
     except ValueError:
-        if value.lower() in ('true', 't'):
+        if value.lower() in ('true', 't', 'y', 'yes'):
             return True
-        if value.lower() in ('false', 'f'):
+        if value.lower() in ('false', 'f', 'n', 'no'):
             return False
     return default
 
@@ -41,9 +42,9 @@ def to_int(name, default):
         raise ImproperlyConfigured('environ variable %s must be an integer' % name)
 
 
-DEBUG = 'DEBUG' in os.environ
-DEBUG_PROPAGATE_EXCEPTIONS = 'DEBUG_PROPAGATE_EXCEPTIONS' in os.environ
-USE_DEBUG_TOOLBAR = 'USE_DEBUG_TOOLBAR' in os.environ
+DEBUG = to_boolean('DEBUG', default=False)
+DEBUG_PROPAGATE_EXCEPTIONS = to_boolean('DEBUG_PROPAGATE_EXCEPTIONS', default=False)
+USE_DEBUG_TOOLBAR = to_boolean('USE_DEBUG_TOOLBAR', default=False)
 TEMPLATE_DEBUG = DEBUG
 
 BASE_DIR = os.path.dirname(__file__)
@@ -121,6 +122,7 @@ MIDDLEWARE_CLASSES = (
     'django.middleware.transaction.TransactionMiddleware',
     'authentic2.idp.middleware.DebugMiddleware',
     'authentic2.middleware.CollectIPMiddleware',
+    'authentic2.middleware.StoreRequestMiddleware',
 )
 
 A2_OPENED_SESSION_COOKIE_DOMAIN = os.environ.get('A2_OPENED_SESSION_COOKIE_DOMAIN')
@@ -136,13 +138,19 @@ ROOT_URLCONF = 'authentic2.urls'
 TEMPLATE_DIRS = (
         os.path.join(VAR_DIR, 'templates'),
         os.path.join(BASE_DIR, 'templates'),
+        os.path.join(BASE_DIR, 'manager', 'vendor', 'gadjo', 'gadjo', 'templates'),
 )
 
 STATICFILES_DIRS = (
         os.path.join(VAR_DIR, 'extra-static'),
         os.path.join(BASE_DIR, 'static'),
+        os.path.join(BASE_DIR, 'manager', 'vendor', 'gadjo', 'gadjo', 'static'),
 )
 
+STATICFILES_DIRS = tuple(staticfile_dir for staticfile_dir in STATICFILES_DIRS if os.path.isdir(staticfile_dir))
+
+STATICFILES_FINDERS = global_settings.STATICFILES_FINDERS + ('gadjo.finders.XStaticFinder',)
+
 if os.environ.get('TEMPLATE_DIRS'):
     TEMPLATE_DIRS = tuple(os.environ['TEMPLATE_DIRS'].split(':')) + TEMPLATE_DIRS
 
@@ -184,6 +192,7 @@ INSTALLED_APPS = (
     'authentic2.disco_service',
     'authentic2.manager',
     'authentic2',
+    'gadjo',
 )
 
 INSTALLED_APPS = plugins.register_plugins_installed_apps(INSTALLED_APPS)
@@ -195,16 +204,17 @@ MESSAGE_STORAGE = 'django.contrib.messages.storage.session.SessionStorage'
 # authentication
 AUTHENTICATION_BACKENDS = (
     'authentic2.backends.ModelBackend',
+    'authentic2.backends.LDAPBackend',
 )
 AUTHENTICATION_BACKENDS = plugins.register_plugins_authentication_backends(
         AUTHENTICATION_BACKENDS)
 
 # sessions
-SESSION_EXPIRE_AT_BROWSER_CLOSE =  'SESSION_EXPIRE_AT_BROWSER_CLOSE' in os.environ
+SESSION_EXPIRE_AT_BROWSER_CLOSE =  to_boolean('SESSION_EXPIRE_AT_BROWSER_CLOSE', default=False)
 SESSION_COOKIE_AGE = int(os.environ.get('SESSION_COOKIE_AGE', 36000)) # one day of work
 SESSION_COOKIE_NAME = os.environ.get('SESSION_COOKIE_NAME', 'sessionid')
 SESSION_COOKIE_PATH = os.environ.get('SESSION_COOKIE_PATH', '/')
-SESSION_COOKIE_SECURE = 'SESSION_COOKIE_SECURE' in os.environ
+SESSION_COOKIE_SECURE = to_boolean('SESSION_COOKIE_SECURE', default=False)
 if 'SESSION_ENGINE' in os.environ:
     SESSION_ENGINE = os.environ['SESSION_ENGINE']
 
@@ -214,7 +224,7 @@ EMAIL_HOST_USER = os.environ.get('EMAIL_HOST_USER', '')
 EMAIL_HOST_PASSWORD = os.environ.get('EMAIL_HOST_PASSWORD', '')
 EMAIL_PORT = int(os.environ.get('EMAIL_PORT', 25))
 EMAIL_SUBJECT_PREFIX = os.environ.get('EMAIL_SUBJECT_PREFIX', '[Authentic]')
-EMAIL_USE_TLS = 'EMAIL_USE_TLS' in os.environ
+EMAIL_USE_TLS = to_boolean('EMAIL_USE_TLS', default=False)
 SERVER_EMAIL = os.environ.get('SERVER_EMAIL', 'root@localhost')
 DEFAULT_FROM_EMAIL = os.environ.get('DEFAULT_FROM_EMAIL', 'webmaster@localhost')
 
@@ -225,8 +235,8 @@ USE_X_FORWARDED_HOST = 'USE_X_FORWARDED_HOST' in os.environ
 if 'SECURE_PROXY_SSL_HEADER' in os.environ:
     SECURE_PROXY_SSL_HEADER = os.environ['SECURE_PROXY_SSL_HEADER'].split(':', 1)
 LOGIN_REDIRECT_URL = os.environ.get('LOGIN_REDIRECT_URL', '/')
-LOGIN_URL = os.environ.get('LOGIN_URL', '/login')
-LOGOUT_URL = os.environ.get('LOGOUT_URL', '/accounts/logout')
+LOGIN_URL = os.environ.get('LOGIN_URL', '/login/')
+LOGOUT_URL = os.environ.get('LOGOUT_URL', '/logout/')
 
 if 'INTERNAL_IPS' in os.environ:
     INTERNAL_IPS = os.environ['INTERNAL_IPS'].split(':')
@@ -239,21 +249,22 @@ DEBUG_TOOLBAR_CONFIG = {'INTERCEPT_REDIRECTS': False}
 
 # Authentic2 settings
 
-DISCO_SERVICE = 'DISCO_SERVICE' in os.environ
-DISCO_USE_OF_METADATA = 'DISCO_USE_OF_METADATA' in os.environ
+DISCO_SERVICE = to_boolean('DISCO_SERVICE', default=False)
+DISCO_USE_OF_METADATA = to_boolean('DISCO_USE_OF_METADATA', default=False)
 
 DISCO_SERVICE_NAME = os.environ.get('DISCO_SERVICE_NAME', "http://www.identity-hub.com/disco_service/disco")
 DISCO_RETURN_ID_PARAM = "entityID"
-SHOW_DISCO_IN_MD = 'SHOW_DISCO_IN_MD' in os.environ
+SHOW_DISCO_IN_MD = to_boolean('SHOW_DISCO_IN_MD', default=False)
 
 ###########################
 # Authentication settings
 ###########################
 
+AUTH_FRONTENDS = plugins.register_plugins_auth_frontends((
+    'authentic2.auth_frontends.LoginPasswordBackend',))
+
 # Only RSA private keys are currently supported
-AUTH_FRONTENDS = ( 'authentic2.auth_frontends.LoginPasswordBackend',)
-AUTH_FRONTENDS = plugins.register_plugins_auth_frontends(AUTH_FRONTENDS)
-SSLAUTH_CREATE_USER = 'SSLAUTH_CREATE_USER' in os.environ
+SSLAUTH_CREATE_USER = to_boolean('SSLAUTH_CREATE_USER', default=False)
 AUTHENTICATION_EVENT_EXPIRATION = int(os.environ.get('AUTHENTICATION_EVENT_EXPIRATION', 3600*24*7))
 
 #############################
@@ -319,7 +330,7 @@ TKX6tp6oI+7MIJE6ySZ0cBqOiydAkBePZhu57j6ToBkTa0dbHjn1WA==
 # Can be none, sp, idp or both
 SAML_METADATA_AUTOLOAD = os.environ.get('SAML_METADATA_AUTOLOAD', 'none')
 
-PUSH_PROFILE_UPDATES = 'PUSH_PROFILE_UPDATES' in os.environ
+PUSH_PROFILE_UPDATES = to_boolean('PUSH_PROFILE_UPDATES', default=False)
 
 
 if 'PASSWORD_HASHERS' in os.environ:
@@ -351,7 +362,7 @@ else:
 CACHE_DIR = os.path.join('/var/cache/', PROJECT_NAME)
 if not os.access(CACHE_DIR, os.W_OK):
     CACHE_DIR = os.path.join(PROJECT_DIR, 'cache')
-    print 'Cannot access global cache path, using in project cache directory', CACHE_DIR
+    warnings.warn('Cannot access global cache path, using in project cache directory %s' %  CACHE_DIR)
     if not os.path.isdir(CACHE_DIR):
         os.makedirs(CACHE_DIR)
 
@@ -392,11 +403,11 @@ SOUTH_TESTS_MIGRATE = False
 ############################
 # Registration
 ############################
-A2_ACCEPT_EMAIL_AUTHENTICATION = to_boolean('A2_ACCEPT_EMAIL_AUTHENTICATION')
-A2_CAN_RESET_PASSWORD = to_boolean('A2_CAN_RESET_PASSWORD')
-A2_REGISTRATION_CAN_DELETE_ACCOUNT = to_boolean('A2_REGISTRATION_CAN_DELETE_ACCOUNT')
+A2_ACCEPT_EMAIL_AUTHENTICATION = to_boolean('A2_ACCEPT_EMAIL_AUTHENTICATION', default=True)
+A2_CAN_RESET_PASSWORD = to_boolean('A2_CAN_RESET_PASSWORD', default=True)
+A2_REGISTRATION_CAN_DELETE_ACCOUNT = to_boolean('A2_REGISTRATION_CAN_DELETE_ACCOUNT', default=True)
 A2_REGISTRATION_EMAIL_IS_UNIQUE = to_boolean('A2_REGISTRATION_EMAIL_IS_UNIQUE', default=False)
-REGISTRATION_OPEN = to_boolean('REGISTRATION_OPEN')
+REGISTRATION_OPEN = to_boolean('REGISTRATION_OPEN', default=True)
 ACCOUNT_ACTIVATION_DAYS = to_int('ACCOUNT_ACTIVATION_DAYS', 3)
 PASSWORD_RESET_TIMEOUT_DAYS = to_int('PASSWORD_RESET_TIMEOUT_DAYS', 3)
 
@@ -408,10 +419,11 @@ ADMIN_TOOLS_INDEX_DASHBOARD = 'authentic2.dashboard.CustomIndexDashboard'
 ADMIN_TOOLS_APP_INDEX_DASHBOARD = 'authentic2.dashboard.CustomAppIndexDashboard'
 ADMIN_TOOLS_MENU = 'authentic2.menu.CustomMenu'
 
-AUTH_OPENID = 'AUTH_OPENID' in os.environ
-AUTH_SSL = 'AUTH_SSL' in os.environ
-IDP_SAML2 = 'IDP_SAML2' in os.environ
-IDP_OPENID = 'IDP_OPENID' in os.environ
+A2_AUTH_PASSWORD_ENABLE = to_boolean('AUTH_PASSWORD', default=True)
+SSLAUTH_ENABLE = to_boolean('AUTH_SSL', 
+        default=to_boolean('SSLAUTH_ENABLE', default=False))
+A2_IDP_SAML2_ENABLE = to_boolean('IDP_SAML2', default=False)
+IDP_OPENID = to_boolean('IDP_OPENID', default=False)
 
 # extract any key starting with setting
 for key in os.environ:
@@ -468,6 +480,9 @@ except ImportError, e:
     if 'local_settings' in e.args[0]:
         pass
 
+if 'IDP_SAML2' in globals():
+    A2_IDP_SAML2_ENABLE = IDP_SAML2
+
 from . import fix_user_model
 
 LOGGING = {
@@ -480,11 +495,11 @@ LOGGING = {
     },
     'formatters': {
         'verbose': {
-            'format': '[%(asctime)s] %(levelname)s %(name)s: %(message)s',
+            'format': '[%(asctime)s] %(levelname)s %(name)s.%(funcName)s: %(message)s',
             'datefmt': '%Y-%m-%d %a %H:%M:%S'
         },
         'syslog': {
-            'format': '%(levelname)s %(name)s: %(message)s',
+            'format': '%(levelname)s %(name)s.%(funcName)s: %(message)s',
         },
     },
     'handlers': {
@@ -522,28 +537,30 @@ LOGGING = {
             'handlers': ['null'],
             'propagate': True,
         },
+        'lasso': {
+                'handlers': ['mail_admins', 'syslog'],
+                'level': 'INFO',
+                'propagate': False,
+        },
         '': {
                 'handlers': ['mail_admins', 'syslog'],
                 'level': 'INFO',
-        }
+        },
     },
 }
 
 if DEBUG and not DEBUG_LOG:
-    LOGGING['loggers']['']['handlers'] += ['console']
     LOGGING['loggers']['']['level'] = 'DEBUG'
 
+if to_boolean('CONSOLE_LOG'):
+    LOGGING['loggers']['']['handlers'] += ['console']
+
 if DEBUG_LOG:
     domains = DEBUG_LOG.split()
     for domain in domains:
         logger = LOGGING['loggers'].setdefault(domain, {
-                'handlers': ['mail_admins', 'syslog'],
                 'level': 'DEBUG',
         })
-        if 'syslog' not in logger['handlers']:
-            logger['handlers'] += ['syslog']
-        if 'console' not in logger['handlers']:
-            logger['handlers'] += ['console']
         logger['level'] = 'DEBUG'
 
 if SENTRY_DSN is not None:
@@ -564,29 +581,14 @@ if SENTRY_DSN is not None:
 if USE_DEBUG_TOOLBAR:
     try:
         import debug_toolbar
+        DEBUG_TOOLBAR_PATCH_SETTINGS = False
         MIDDLEWARE_CLASSES += ('debug_toolbar.middleware.DebugToolbarMiddleware',)
         INSTALLED_APPS += ('debug_toolbar',)
     except ImportError:
-        print "Debug toolbar missing, not loaded"
-
-if AUTH_OPENID:
-    INSTALLED_APPS += ('authentic2.auth2_auth.auth2_openid', 'django_authopenid',)
-    AUTH_FRONTENDS += ('authentic2.auth2_auth.auth2_openid.backend.OpenIDFrontend',)
-
-if AUTH_SSL:
-    AUTHENTICATION_BACKENDS += ('authentic2.auth2_auth.auth2_ssl.backend.SSLBackend',)
-    AUTH_FRONTENDS += ('authentic2.auth2_auth.auth2_ssl.frontend.SSLFrontend',)
-    INSTALLED_APPS += ('authentic2.auth2_auth.auth2_ssl',)
-
-if IDP_SAML2:
-    IDP_BACKENDS += ('authentic2.idp.saml.backend.SamlBackend',)
+        warnings.warn("Debug toolbar missing, not loaded")
 
 if IDP_OPENID:
     # RESTRICT_OPENID_RP = ["http://rp.example.com", ] # orequest.trust_root
     INSTALLED_APPS += ('authentic2.idp.idp_openid',)
     TEMPLATE_CONTEXT_PROCESSORS += ('authentic2.idp.idp_openid.context_processors.openid_meta',)
     # OPENID_ACTIONS = {"http://rp.example.com" : 'my-template.html', }
-
-if LDAP_AUTH_SETTINGS:
-    AUTHENTICATION_BACKENDS = ('authentic2.backends.LDAPBackend',) + AUTHENTICATION_BACKENDS
-
