@@ -24,10 +24,11 @@ from django.contrib.auth import logout as auth_logout
 from django.contrib.auth import REDIRECT_FIELD_NAME
 from django.http import (HttpResponseRedirect, HttpResponseForbidden,
     HttpResponse)
+from django.core.exceptions import PermissionDenied
 from django.views.decorators.csrf import csrf_protect
 from django.views.decorators.cache import never_cache
 from django.contrib.auth.decorators import login_required
-from django.db.models.fields import FieldDoesNotExist
+from django.db.models import FieldDoesNotExist
 
 
 # FIXME: this decorator has nothing to do with an idp, should be moved in the
@@ -99,7 +100,7 @@ class EditProfile(UpdateView):
                             self.request.user, sp_id)
 
     def form_valid(self, form):
-        if settings.PUSH_PROFILE_UPDATES:
+        if app_settings.PUSH_PROFILE_UPDATES:
             thread.start_new_thread(self.push_attributes, ())
         return super(EditProfile, self).form_valid(form)
 
@@ -108,8 +109,8 @@ class EditProfile(UpdateView):
         kwargs['prefix'] = 'edit-profile'
         return kwargs
 
-
-edit_profile = decorators.prevent_access_to_transient_users(EditProfile.as_view())
+edit_profile = decorators.setting_enabled('A2_PROFILE_CAN_EDIT_PROFILE')(
+    decorators.prevent_access_to_transient_users(EditProfile.as_view()))
 
 
 def su(request, username, redirect_url='/'):
@@ -174,7 +175,8 @@ class EmailChangeView(FormView):
                   'link contained inside.'))
         return super(EmailChangeView, self).form_valid(form)
 
-email_change = decorators.prevent_access_to_transient_users(EmailChangeView.as_view())
+email_change = decorators.setting_enabled('A2_PROFILE_CAN_CHANGE_EMAIL')(
+    decorators.prevent_access_to_transient_users((EmailChangeView.as_view())))
 
 class EmailChangeVerifyView(TemplateView):
     def get(self, request, *args, **kwargs):
@@ -363,11 +365,13 @@ class ProfileView(TemplateView):
             profile.append((title, map(unicode, value)))
         # Credentials management
         blocks = [ frontend.profile(request) for frontend in frontends \
-                if hasattr(frontend, 'profile') ]
+                if hasattr(frontend, 'profile') and frontend.enabled() ]
         ctx.update({
             'frontends_block': blocks,
             'profile': profile,
             'allow_account_deletion': app_settings.A2_REGISTRATION_CAN_DELETE_ACCOUNT,
+            'allow_profile_edit': app_settings.A2_PROFILE_CAN_EDIT_PROFILE,
+            'allow_email_change': app_settings.A2_PROFILE_CAN_CHANGE_EMAIL,
         })
         return ctx
 
@@ -406,8 +410,11 @@ def redirect_to_logout(request, next_page='/'):
 
 
 def login_password_profile(request):
-    return render_to_string('auth/login_password_profile.html', {},
-            RequestContext(request))
+    can_change_password = (app_settings.A2_REGISTRATION_CAN_CHANGE_PASSWORD
+                           and request.user.has_usable_password)
+    return render_to_string('auth/login_password_profile.html',
+                            {'can_change_password' : can_change_password},
+                            RequestContext(request))
 
 
 def redirect_to_login(request, next=None, nonce=None, keep_qs=False):
@@ -420,7 +427,7 @@ def redirect_to_login(request, next=None, nonce=None, keep_qs=False):
     qs = { REDIRECT_FIELD_NAME: next }
     if nonce is not None:
         qs.update({ constants.NONCE_FIELD_NAME: nonce })
-    return HttpResponseRedirect('/login?%s' % urlencode(qs))
+    return HttpResponseRedirect('%s?%s' % (reverse('auth_login'), urlencode(qs)))
 
 
 
